package com.mjitech.iot;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.timzaak.ucloud.iot.UIotServerClient;
import com.timzaak.ucloud.iot.model.CreateUIoTCoreDeviceResult;
import com.timzaak.ucloud.iot.model.GetDevicePropertyResult;
import com.timzaak.ucloud.iot.model.GetUIoTCoreDeviceInfoResult;
import com.timzaak.ucloud.iot.model.SendDeviceCommandResult;

import javax.validation.constraints.Null;

public class IotClient {
    private final UIotServerClient client;
    private final ObjectMapper objectMapper;

    public IotClient(UIotServerClient client, ObjectMapper objectMapper) {
        this.client = client;
        this.objectMapper = objectMapper;
    }

    /*
    {
      storeId: $number,
      state: $number, // 当前机器状态
      images: [$string], // 当前运行的 docker 镜像
      networks: [{
        "mac": $string, //mac 地址
        "ip": $string //
      }]
      electronDoor: $number,//电柜门状态(0:开，1:关)
      aDoor: $number,//a门状态(0:开，1:关)
      bDoor: $number,//b门状态(0:开，1:关)
    }
    获取设备属性， 结果是一个 json 字符串，需要自行转译
     */

    public GetDevicePropertyResult getDeviceInfo(String deviceSN) {
        try {
            return client.getDeviceProperty(deviceSN);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    // 更改服务状态， 例如上货的时候，要先让机器进入维护状态，才能开门
    public SendDeviceCommandResult setState(String deviceSN, int deviceStatus) {
        //"Status Only support this three"
        assert (deviceStatus == DeviceStatus.Free || deviceStatus == DeviceStatus.InRepair || deviceStatus == DeviceStatus.UpdateInReapire);
        ObjectNode node = objectMapper.createObjectNode();
        node.put("stateId", deviceStatus);
        node.put("isForce", 0);
        try {
            return client.sendDeviceCommandAsync(deviceSN, "SetStatus", node.toString());
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    // 刷新浏览器
    public SendDeviceCommandResult reloadBrowser(String deviceSN) {
        try {
            return client.sendDeviceCommandAsync(deviceSN, "ReloadBrowser", "{}");
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    // 开门
    public SendDeviceCommandResult openDoor(String deviceSN, int doorId, int employeeId) {
        //只支持这三种门
        assert(doorId == DoorId.ADoor || doorId == DoorId.BDoor || doorId == DoorId.ElectronDoor);
        ObjectNode node = objectMapper.createObjectNode();
        node.put("doorId", doorId);
        node.put("isForce", 0);
        node.put("employeeId",employeeId);
        try {
            return client.sendDeviceCommandAsync(deviceSN, "OpenDoor", node.toString());
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    //添加设备
    public CreateUIoTCoreDeviceResult addDevice(String deviceSN, String Description) {
        try {
            return client.CreateUIoTCoreDevice(deviceSN, Description);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    //获取设备信息(带设备密码)
    public GetUIoTCoreDeviceInfoResult getDeviceData(String deviceSN) {
        try {
            return client.GetUIoTCoreDeviceInfo(deviceSN);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    //重启服务
    public SendDeviceCommandResult reloadServer(String deviceSN) {
        try {
            return client.sendDeviceCommandAsync(deviceSN, "ReloadServer", "{}");
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    //取货
    public SendDeviceCommandResult NewSkuPass(String deviceSN, int shelfNumber, int x, int y, int x1, int y1) {
        ObjectNode node = objectMapper.createObjectNode();
        node.put("shelfNumber", shelfNumber);
        node.put("x", x);
        node.put("y", y);
        node.put("x1", x1);
        node.put("y1", y1);
        try {
            return client.sendDeviceCommandAsync(deviceSN, "NewSkuPass", node.toString());
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    //开货仓门
    public SendDeviceCommandResult OpenGoodDoor(String deviceSN) {
        try {
            return client.sendDeviceCommandAsync(deviceSN, "OpenGoodDoor", "{}");
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }


}
