package com.timzaak.ucloud.iot;

import cn.ucloud.common.client.DefaultUcloudClient;
import cn.ucloud.common.client.UcloudClient;
import cn.ucloud.common.pojo.Account;
import cn.ucloud.common.pojo.BaseResponseResult;
import cn.ucloud.common.pojo.UcloudConfig;
import com.timzaak.ucloud.iot.model.*;
import org.apache.commons.codec.binary.Base64;


// api 文档 https://docs.ucloud.cn/uiot-core/api_guide/api_list
public class UIotServerClient {
    private final UcloudClient client;

    private final String productSN;
    //上海2
    private static final String Region = "cn-sh2";

    public UIotServerClient(String privateKey, String publicKey, String productSN) {
        this.productSN = productSN;
        client = new DefaultUcloudClient(new UcloudConfig(
                new Account(privateKey, publicKey)
        ));
    }


    public BaseResponseResult publishMessage(String topicFullName, String messageContent, int qos) throws Exception {
        PublishMessage message = new PublishMessage(
                Region, productSN, topicFullName
        );
        message.setMessageContent(Base64.encodeBase64String(messageContent.getBytes()));
        message.setQos(qos);
        return client.doAction(
                message
                , BaseResponseResult.class);
    }

    public BaseResponseResult publishMessage(String topicFullName, String messageContent) throws Exception {
        return this.publishMessage(topicFullName, messageContent, 0);
    }

    public BaseResponseResult broadcastMessage(String topicFullName, String messageContent) throws Exception {
        BroadcastMessage message = new BroadcastMessage(
                Region, productSN, topicFullName
        );

        message.setMessageContent(Base64.encodeBase64String(messageContent.getBytes()));
        return client.doAction(
                message
                , BaseResponseResult.class);
    }

    public GetDevicePropertyResult getDeviceProperty(String deviceSN) throws Exception {
        GetDevicePropertyRequest req = new GetDevicePropertyRequest();
        req.setDeviceSN(deviceSN);
        req.setProductSN(productSN);
        req.setRegion(Region);
        return (GetDevicePropertyResult) client.doAction(req, GetDevicePropertyResult.class);
    }

    public BaseResponseResult setDeviceProperty(String deviceSN, String property, boolean desired) throws Exception {
        SetDevicePropertyRequest req = new SetDevicePropertyRequest();
        req.setRegion(Region);
        req.setProductSN(productSN);
        req.setDeviceSN(deviceSN);
        req.setProperty(Base64.encodeBase64String(property.getBytes()));
        req.setDesired(desired);
        return client.doAction(req, BaseResponseResult.class);
    }


    public SendDeviceCommandResult sendDeviceCommandAsync(String deviceSN, String identifier, String input) throws Exception {
        SendDeviceCommandRequest req = new SendDeviceCommandRequest();
        req.setRegion(Region);
        req.setProductSN(productSN);
        req.setDeviceSN(deviceSN);
        req.setIdentifier(identifier);
        req.setInput(Base64.encodeBase64String(input.getBytes()));
        req.setMethod("async");
        return (SendDeviceCommandResult) client.doAction(req, SendDeviceCommandResult.class);
    }

    public SendDeviceCommandResult sendDeviceCommandSync(String deviceSN, String identifier, String input) throws Exception {
        SendDeviceCommandRequest req = new SendDeviceCommandRequest();
        req.setRegion(Region);
        req.setProductSN(productSN);
        req.setDeviceSN(deviceSN);
        req.setIdentifier(identifier);
        req.setInput(Base64.encodeBase64String(input.getBytes()));
        req.setMethod("sync");
        req.setTimeout(5);
        return (SendDeviceCommandResult) client.doAction(req, SendDeviceCommandResult.class);
    }

    public CreateUIoTCoreDeviceResult CreateUIoTCoreDevice(String deviceSN, String Description) throws Exception {
        CreateUIoTCoreDeviceRequest req = new CreateUIoTCoreDeviceRequest();
        req.setRegion(Region);
        req.setProductSN(productSN);
        req.setDeviceSN(deviceSN);
        req.setDescription(Description);

        return (CreateUIoTCoreDeviceResult) client.doAction(req, CreateUIoTCoreDeviceResult.class);
    }

    public GetUIoTCoreDeviceInfoResult GetUIoTCoreDeviceInfo(String deviceSN) throws Exception {
        GetUIoTCoreDeviceInfoRequest req = new GetUIoTCoreDeviceInfoRequest();
        req.setRegion(Region);
        req.setProductSN(productSN);
        req.setDeviceSN(deviceSN);

        return (GetUIoTCoreDeviceInfoResult) client.doAction(req, GetUIoTCoreDeviceInfoResult.class);
    }


}
