
import com.timzaak.jackson.KtorJacksonWrapper.jackson
import com.typesafe.config.ConfigFactory
import di.thirdMockDI
import io.ktor.application.Application
import io.ktor.application.call
import io.ktor.application.install
import io.ktor.features.CORS
import io.ktor.features.CallLogging
import io.ktor.features.ContentNegotiation
import io.ktor.features.DefaultHeaders
import io.ktor.http.ContentType
import io.ktor.http.HttpMethod
import io.ktor.response.respondText
import io.ktor.routing.Routing
import io.ktor.routing.get
import io.ktor.server.engine.commandLineEnvironment
import io.ktor.server.engine.embeddedServer
import io.ktor.server.netty.Netty
import org.koin.dsl.module
import org.koin.ktor.ext.Koin
import routes.mockRoute

fun Application.mockMain() {
    val config = ConfigFactory.load()
    install(CORS) {
        method(HttpMethod.Options)
        anyHost()
        allowCredentials = true
        allowNonSimpleContentTypes = true
    }

    install(DefaultHeaders)
    install(CallLogging)
    install(Koin) {
        modules(
            module(createdAtStart = true) {
                single { coroutineContext }
            },
            thirdMockDI(config)
        )
    }

    install(ContentNegotiation) {
        jackson()
    }

    install(Routing) {
        get("/") {
            call.respondText("Hello World", ContentType.Text.Html)
        }
        mockRoute(config)
    }
}