
import com.mjitech.api.BaseAPIImpl
import com.timzaak.jackson.KtorJacksonWrapper.jackson
import com.timzaak.ws.ConnectionManager
import com.typesafe.config.ConfigFactory
import config.AppConfig
import di.*
import io.ktor.application.*
import io.ktor.client.HttpClient
import io.ktor.client.engine.apache.Apache
import io.ktor.client.features.HttpTimeout
import io.ktor.client.features.logging.DEFAULT
import io.ktor.client.features.logging.LogLevel
import io.ktor.client.features.logging.Logger
import io.ktor.client.features.logging.Logging
import io.ktor.features.CORS
import io.ktor.features.CallLogging
import io.ktor.features.ContentNegotiation
import io.ktor.features.DefaultHeaders
import io.ktor.http.ContentType
import io.ktor.http.HttpMethod
import io.ktor.response.respondText
import io.ktor.routing.Routing
import io.ktor.routing.get
import io.ktor.server.engine.commandLineEnvironment
import io.ktor.server.engine.embeddedServer
import io.ktor.server.netty.Netty
import io.ktor.util.KtorExperimentalAPI
import io.ktor.websocket.WebSockets
import mu.KotlinLogging
import org.koin.dsl.module
import org.koin.ktor.ext.Koin
import routes.*
import routes.pos.posRoutes
import routes.test.testRoutes

private val logger = KotlinLogging.logger {}
@KtorExperimentalAPI
fun Application.main() {
    val config = ConfigFactory.load()
    val appConfig = AppConfig(config)
    val httpClient = HttpClient(Apache) {
        install(HttpTimeout)
//         install(Logging) {
//             logger = Logger.DEFAULT
//             level = LogLevel.ALL
//         }
    }
    val connectionManager = ConnectionManager()
    val baseAPIImpl = BaseAPIImpl(
        appConfig, httpClient,coroutineContext
    )
    install(CORS) {
        method(HttpMethod.Options)
        anyHost()
        allowCredentials = true
        allowNonSimpleContentTypes = true
    }

    install(DefaultHeaders)
    install(CallLogging)
    install(WebSockets)
    install(Koin) {
        modules(
            module(createdAtStart = true) {
                single { config }
                single { coroutineContext }
                single { appConfig }
                single { connectionManager }
                single { httpClient }
                single { baseAPIImpl }
            },
            thirdDI(config),
            thirdMockDI(config),
            apiDI(),
            serviceDI(config)
        )
    }
    lazyInitService()

    install(ContentNegotiation) {
        jackson()
    }

    install(Routing) {
        get("/") {
            call.respondText("华沁", ContentType.Text.Html)
        }
        upperRouters(appConfig)
        posRoutes(appConfig)
        redirectToServer(baseAPIImpl)
        websocketRoute(appConfig)
        if(appConfig.environment != "product"){
            testRoutes()
            mockRoute(config)
        }
    }
    //TODO: 监听 服务挂掉时间，同时清理 MockDownServer + DownClient + ScanCodeClient + MockScanCodeServer + Mqtt

    //cleanServer()

}

fun main(args: Array<String>) {
    embeddedServer(Netty, commandLineEnvironment(args)).start(wait = true)
}
