package com.mjitech.api

import com.fasterxml.jackson.databind.node.ArrayNode
import com.fasterxml.jackson.databind.node.ObjectNode
import com.mjitech.api.model.FailSku
import com.mjitech.api.model.Order
import com.mjitech.api.model.SKU
import com.timzaak.jackson.KtorJacksonWrapper
import com.timzaak.jackson.jsObject
import com.timzaak.ktor.params
import config.AppConfig
import io.ktor.http.Parameters
import io.ktor.http.parametersOf
import io.ktor.http.plus

class OrderAPI(private val baseAPIImpl: BaseAPIImpl) : API by baseAPIImpl {
    suspend fun getOrderDetailByTokenId(takingNumber: String): Result<ObjectNode> {
        val data = parametersOf("takingNumber", takingNumber)
        return post("/machine_api/get_order_detail_by_takingnumber.action", data).jsonResult()
    }
    suspend fun updateOrderStatus(orderNumber: String, status: Int): Result<Unit> {
        val data = parametersOf("orderNumber", orderNumber) + parametersOf("status", status.toString())
        return post("/machine_api/update_order_status.action", data).result()
    }

    suspend fun getExchangeSku(
        outBatchId: Int,
        outBatchSkuId: Int,
        orderNumber: String,
        skuId: Int
    ): Result<Pair<Boolean, ExchangeSKUResponse>> {
        val data = params(
            "outBatchId" to outBatchId.toString(),
            "outBatchSkuId" to outBatchSkuId.toString(),
            "orderNumber" to orderNumber,
            "skuId" to skuId.toString()
        )
        return post("/machine_api/get_exchanged_sku.action", data).resultWithB(null)
    }

    //pos机支付完成订单
    suspend fun getOrderForCardPay(data: String): Result<Pair<Boolean, OrderForCardPayResponse>> {
        val queryData = (KtorJacksonWrapper.jacksonMapper.readTree(data) as ObjectNode).fields().asSequence()
                .fold(Parameters.Empty) { acc, i ->
                    when (val v = i.value) {
                        is ArrayNode -> {
                            acc + v.fold(Parameters.Empty) { acc1, i1 ->
                                acc1 + parametersOf(i.key, i1.asText())
                            }
                        }
                        else -> {
                            acc + parametersOf(i.key, i.value.asText())
                        }
                    }
                }
        return post("/machine_api/get_order_detail_for_card_pay.action", queryData).resultWithB()
    }

    //pos机连接
    suspend fun connectPos(cardPayBusinessType:String,order: CardPayOrder,printContent:String): Result<ObjectNode> {
        val posData = params(
                "BUSINESS_ID" to cardPayBusinessType,
                "AMOUNT" to order.totalPrice,
                "TRANS_CHECK" to order.orderNumber,
                "ORDER_NO" to order.orderNumber,
                "PAGE_APPEND_CONTENT" to printContent
        )

        return posPost(posData).jsonResult()
    }

    //pos机支付完成订单
    suspend fun updateCardPay(
            orderNumber: String,
            traceNumber: String,
            cardNumber: String
            ): Result<ObjectNode> {
        val data = params(
                "orderNumber" to orderNumber,
                "cardNumber" to cardNumber,
                "traceNumber" to traceNumber
        )
        return post("/machine_api/update_order_card_pay_succ.action", data).jsonResult()
    }

    // 上报错误
    suspend fun reportError(
        errorType: Int,
        errorInfo: String,
        shouldRefund: Boolean,
        outBatchId: Int,
        outBatchSkuId: Int
    ): Result<Pair<Boolean, ExchangeSKUResponse>> {
        val data = params(
            "errorType" to errorType.toString(),
            "errorInfo" to errorInfo,
            "shouldRefund" to shouldRefund.toString(),
            "outBatchId" to outBatchId.toString(),
            "outBatchSkuId" to outBatchSkuId.toString()
        )
        return post("/machine_api/upload_error.action", data).resultWithB(null)
    }

    // 上报下位机错误信息
    suspend fun reportDownError(
            superType: Int,
            subType: Int
    ): Result<Unit> {
        val data = params(
                "superType" to superType.toString(),
                "subType" to subType.toString()
        )
        return post("/maxbox_pc/local_api/upload_error_detail.action", data).result()
    }
}

data class ExchangeSKUResponse(
    val error_message: String?,
    val originalSku: SKU,
    val error_code: Int?,
    val is_succ: Boolean,
    val sku: SKU?,
    val removePresent: List<Int>
)

data class NotifySKUMissResponse(
    val skus: List<FailSku>
)

data class OrderForCardPayResponse(
        val order: CardPayOrder,
        val printContent: String

)

data class CardPayOrder(
        val orderNumber: String,
        val status: Int,
        val totalPrice: String
)

data class PosResponse(
        val rejcode: String,
        val cardno: String,
        val transTicketNo: String
)