package com.mjitech.api.model

data class Order(
    val batches: MutableList<OrderBatch>,
    val orderNumber: String,
    val id: Int
) {

    fun replaceSku(outBatchSkuIds: List<Int>, batchStep:Int): Order {
        for (id in outBatchSkuIds) {
            batches[batchStep].skus.filter { oldsku -> oldsku.outBatchSkuId != id }
        }
        return this.copy(batches = batches)
    }

    fun getBatchByStep(step: Int): OrderBatch? {
        return if(batches.size == step) {
            null
        } else {
            batches.get(step)
        }
    }

    fun getSKUByStep(batchStep: Int, step: Int): SKU? {
        return getBatchByStep(batchStep)?.skus?.get(step)
    }
}

data class OrderBatch(
    val id: Int,
    val skus: List<SKU>,
    val count: Int,
    val sellOrderId: Int,
    val number: Int,
    val deleteMark: Int,
    val outBatchId: Int,
    val skuCount: Int,
    val status: Int,
    val outStatus: Int
)

object OrderStatus {
    val FinishOrder = 5
    val TakingOrder = 4
    val Paid = 3
}
