package com.mjitech.bash

import com.atlassian.performance.tools.ssh.api.Ssh
import com.atlassian.performance.tools.ssh.api.SshHost
import com.atlassian.performance.tools.ssh.api.auth.PasswordAuthentication
import com.github.salomonbrys.kotson.get
import com.mjitech.iot.DeviceStatus
import com.mjitech.iot.IotClient
import com.timzaak.jackson.KtorJacksonWrapper
import com.timzaak.ucloud.iot.UIotServerClient
import com.typesafe.config.ConfigFactory
import mu.KotlinLogging
import java.io.File
import java.text.SimpleDateFormat
import java.time.Duration
import java.util.*

val log = KotlinLogging.logger{}
class UpgradeUpper {
    val config = ConfigFactory.load()
    val objectMapper = KtorJacksonWrapper.jacksonMapper
    val c = config.getConfig("api.mqttServer")
    val uIotServerClient = UIotServerClient(
            c.getString("privateKey"),
            c.getString("publicKey"),
            config.getString("api.mqtt.productSN")
    )
    val iotClient = IotClient(uIotServerClient,objectMapper)



    fun run(storeId: String, deviceSN: String , imageVersion: String ,ssh:SshContent ) {

        val deviceData = iotClient.getDeviceData(deviceSN)
        val devicePassword = deviceData.deviceInfoSet.password
        val newImage = "uhub.service.ucloud.cn/mjitech/upper:"+imageVersion

        //生成新配置文件application.conf
        productConfigFile(storeId,deviceSN,devicePassword)

        var isFree = false
        //查询设备状态
        while (!isFree){
            val result = iotClient.getDeviceInfo(deviceSN).property
            if(result.get("images").get("Value").toString().equals("[]") ){
                log.warn { "设备镜像为空" }
            }else{
                val machineImage = result.get("images").get("Value").get(0).asString.trimMargin()
                log.info{"machineImage : $machineImage"}
                val state = result.get("state").get("Value").asInt

                isFree = updateDocker(storeId,deviceSN,machineImage,newImage,state,ssh)
            }
            Thread.sleep(3000)
        }
    }


    fun updateDocker(storeId: String , deviceSN: String , machineImage: String , newImage: String , state: Int , ssh: SshContent):Boolean{
        if(state != DeviceStatus.Free){
            return false
        }else{
            //检查镜像是否是最新
            if(machineImage != newImage){
                log.info { "update images" }
                iotClient.setState(deviceSN,DeviceStatus.InRepair)
                val sshHost = SshHost(ssh.IpAddress, ssh.UserName, PasswordAuthentication(ssh.Password),22)
                Ssh(sshHost).newConnection().use { ssh ->

                    //为了保留旧版本application.conf获取当前时间
                    val sdf = SimpleDateFormat() // 格式化时间
                    sdf.applyPattern("yyyyMMddHHmmss")
                    //备份旧配置文件，转入新备份文件
                    ssh.execute("mv /home/mjitech/server/application.$storeId.conf /home/mjitech/server/oldApplication/application.$storeId.${sdf.format(Date())}.conf")
                    val confFile = File("configfile/application.$storeId.conf")
                    ssh.upload(confFile,"/home/mjitech/server/")
                    val resultPull = ssh.execute("docker pull "+newImage, Duration.ofSeconds(999))
                    if(resultPull.isSuccessful()){
                        val resultStop = ssh.execute("docker rm -f upp")
                        if(resultStop.isSuccessful()){
                            val resultRun = ssh.execute("docker run -it --privileged --name upp -v /home/mjitech/server/application.$storeId.conf:/app/application.conf --restart=always -v /var/run/docker.sock:/var/run/docker.sock --network=host -d "+newImage)
                            if(resultRun.isSuccessful()){
                                log.info { "update success" }
                                ssh.execute("docker system prune -a -f")
                                log.info(ssh.execute("docker ps").output)
                            }else{
                                log.info { "run fail" }
                            }
                        }else{
                            log.info { "stop fail" }
                        }
                    }else{
                        log.info { "pull fail" }
                    }
//                        val result = ssh.execute("docker inspect --format '{{json .Config.Image}}' upp")

                }
            }else{
                log.info { "machine image is newest" }
            }
            return true
        }

    }
}



fun main() {

    val r = UpgradeUpper()
        r.run(
            "97",
                "MAXID000097",
                "test-2",
                SshContent("10.243.227.147","mjitech","mjitech123")
    )
}

class SshContent(
        val IpAddress: String,
        val UserName: String,
        val Password: String
)

