package com.mjitech.service.mqtt.command

import com.fasterxml.jackson.annotation.JsonProperty
import com.fasterxml.jackson.databind.JsonNode
import com.fasterxml.jackson.databind.ObjectMapper
import com.mjitech.service.state.entity.Event

interface Command<T : CommandBody<*>> {
    val identifier: String
}

fun JsonNode.getIdentifier(): String {
    return this["Identifier"].asText()
}

fun JsonNode.getRequestID(): String {
    return this["RequestID"].asText()
}
//
// fun convertDataToCommand(data: ByteArray, objectMapper: ObjectMapper):CommandBody<*> {
//    val json = objectMapper.readTree(data)
//    when (json.getIdentifier()) {
//        UploadLog.identifier -> UploadLog.getBody(json, objectMapper)
//    }
//
// }

interface CommandBody<T> {
    val RequestID: String
    val Identifier: String
    val Input: T
}

data class NullBody(override val RequestID: String, override val Identifier: String) : CommandBody<Unit> {
    override val Input: Unit = Unit
}

data class DataBody<T>(
    @get:JsonProperty("RequestID")
    override val RequestID: String,
    @get:JsonProperty("Identifier")
    override val Identifier: String,
    @get:JsonProperty("Input")
    override val Input: T
) : CommandBody<T>

// - [ ] 1. 重启服务
// - [ ] 2. 上传服务日志
// - [ ] 3. 获取服务信息(网络信息,等)
// - [ ] 4. 请求温度
// - [ ] 5. 维护状态下发
// - [ ] 6. 开门, 电子门, A柜门, B柜门

inline fun <reified T : CommandBody<*>> toData(objectMapper: ObjectMapper, data: JsonNode): T {
    return objectMapper.convertValue(data, T::class.java)
}
/*
object GetTemperature : Command<NullBody> {
    override val identifier: String = "GetTemperature"
}
 */

// data class RestartService() :Command
// 不准备做了
object UploadLog : Command<NullBody> {
    override val identifier: String = "UploadLog"
}

object ReloadBrowser : Command<NullBody> {
    override val identifier: String = "ReloadBrowser"
}

object SetStatus : Command<CommandBody<Event.SetState>> {
    override val identifier: String = "SetStatus"
}

object OpenDoor : Command<CommandBody<Event.OpenDoorInRepair>> {
    override val identifier: String = "OpenDoor"
}

object ReloadServer : Command<NullBody> {
    override val identifier: String = "ReloadServer"
}

object NewSkuPass : Command<NullBody> {
    override val identifier: String = "NewSkuPass"
}
object OpenGoodDoor : Command<NullBody> {
    override val identifier: String = "OpenGoodDoor"
}
