package com.mjitech.service.state

import com.fasterxml.jackson.databind.node.ObjectNode

class WebResponseCache {
    data class CacheData(
        val errorOrderInfo: ObjectNode? = null,
        val needReload: Boolean = false
    )

    private lateinit var map: Map<Int, CacheData>

    init {
        reset()
    }

    fun reset() {
        val screens = listOf(7, 27, 65)
        map = screens.map {
            it to CacheData()
        }.toMap()
    }

    fun setOrderInfo(data: ObjectNode) {
        map = map.mapValues { (_, v) ->
            v.copy(errorOrderInfo = data)
        }
    }

    fun setReload() {
        map = map.mapValues { (_, v) ->
            v.copy(needReload = true)
        }
    }

    fun getCacheData(screenSize: Int): CacheData {
        val value = map.getOrDefault(screenSize, CacheData())
        if (value.needReload || value.errorOrderInfo != null) {
            map = map + (screenSize to CacheData())
        }
        return value
    }
}
