package com.mjitech.service.state.entity

import com.mjitech.api.model.FailTaking
import com.mjitech.api.model.Order

import com.mjitech.third.down.entity.server.HeartBeatResponse

sealed class Event {

    sealed class InitEvent:Event() {
        // 下位机初始化成功
        object DownClientInitSuccess:InitEvent()
        // mqtt 始化成功
        object MqttClientInitSuccess:InitEvent()
        //scanCode初始化成功
        //object ScanCodeClientInitSuccess:InitEvent()
    }


    data class DownHeartBeatEvent(val d: HeartBeatResponse) : Event()

    // 扫二维码
    data class ScanCodeEvent(val order: Order, val text: String) : Event()

    // 正在拿货
    object TakingSucc : Event()

    // 取货有遗漏
    object ProductRemain : Event()

    // 取货完成，转为空闲状态
    object TakeToFree : Event()

    // 拿货
    object TakeGoodOk : Event()
    // 换货失败
    object ChangeSkuFailure : Event()
    // 拿货失败
    data class TakeGoodFailure(val downStatus: Byte,val takingWhich:Int) : Event()

    // 开始换货
    data class ChangeSku(val order: Order, val step: Int, val failTaking: FailTaking) : Event()

    // 开门
    object OpeningDoor : Event()

    // 开门成功
    object OpenSuccessDoor : Event()

    // 拿下一个batch
    object DownPrepareForNext : Event()

    // 机器故障
    data class MachineError(val downStatus: Byte) : Event()

    // 故障之后恢复

    object UpdateMachinePc : Event()

    // 维护的时候用得到的
    data class OpenDoorInRepair(val doorId: Int, val isForce: Boolean = false, val employeeId: Int) : Event()

    data class SetState(val stateId: Int, val isForce: Boolean = false) : Event()

    //测试改变状态
    data class TestSetState(val stateId: Int,val order: Order) : Event()

    data class NewSkuPass(val shelfNumber: Int, val x: Int, val y: Int, val x1: Int, val y1: Int) : Event()

    // 下位机心跳丢失
    object DownMachineLost : Event()

    // 下位机心跳重新相应
    object DownMachineReCon : Event()

    // 下位机不安全
    data class DownMachineIsNotSafe(
        val leftHorseDoor: Byte,
        val rightHorseDoor: Byte,
        val replenishmentScreen: Byte
    ) : Event()
}
