package com.mjitech.third.down

import com.mjitech.third.down.entity.CommandRequest
import com.mjitech.third.down.entity.CommandResponse
import com.mjitech.third.down.entity.server.*
import com.mjitech.third.down.entity.toCommand
import com.mjitech.third.down.protocol.getFrameType
import com.mjitech.third.down.protocol.toHumanRead
import mu.KLogger

interface DownClient {
    // should be protected
    val logger: KLogger
    val downClientChannel: DownClientChannel
    fun sendCommand(command: CommandRequest)

    // should be protected
    suspend fun listen(data: ByteArray) {
        val message: CommandResponse? = when (data.getFrameType()) {
            // DownHeartBeat type 和 AKC 一致，写在这里，是预防忘记处理
            ACKResponse.type -> {
                val result = ACKResponse.serializer().toCommand(data)
                result
            }
            HeartBeatResponse.type -> {
                HeartBeatResponse.serializer().toCommand(data)
            }
            OutGoodStatus.type -> {
                //Attenti
                // on, 由于有很多不必要的处理，直接忽略掉,提升性能
                if (data[13].toInt() and 0xff > 11) {
                    null
                } else {
                    OutGoodStatus.serializer().toCommand(data)
                }
            }
            OutGoodDoorStatus.type -> {
                OutGoodDoorStatus.serializer().toCommand(data)
            }
            MachineErrorResponse.type -> {
                MachineErrorResponse.serializer().toCommand(data)
            }
            else -> {
                logger.warn { "receive error thing, type:${data.getFrameType()}, ${data.toHumanRead()}" }
                null
            }
        }

        message?.let {
            downClientChannel.send(it)
        }
    }
}
