package com.mjitech.third.down

import com.mjitech.third.down.entity.CommandRequest
import com.mjitech.third.down.entity.HeartBeatRequest
import com.mjitech.third.down.protocol.toHumanRead
import io.ktor.network.selector.ActorSelectorManager
import io.ktor.network.sockets.ConnectedDatagramSocket
import io.ktor.network.sockets.Datagram
import io.ktor.network.sockets.aSocket
import io.ktor.network.sockets.isClosed
import io.ktor.utils.io.core.ByteReadPacket
import io.ktor.utils.io.core.readBytes
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.isActive
import kotlinx.coroutines.launch
import mu.KLogger
import mu.KotlinLogging
import java.net.InetSocketAddress
import kotlin.coroutines.CoroutineContext

class DownUDPClient(
    private val targetAddress: InetSocketAddress,
    override val downClientChannel: DownClientChannel,
    override val coroutineContext: CoroutineContext
) : DownClient, CoroutineScope {
    override val logger: KLogger = KotlinLogging.logger { }
    private var socket: ConnectedDatagramSocket = aSocket(ActorSelectorManager(Dispatchers.IO)).udp().connect(targetAddress) {
        reuseAddress = true
        //reusePort = true
    }

    init {
        launch {
            while(isActive) {
                try {
                    for (data in socket.incoming) {
                        this@DownUDPClient.listen(data.packet.readBytes())
                    }
                } finally {
                    break
                }
            }
        }
    }


    override fun sendCommand(command: CommandRequest) {
        launch {
            val message = command.toBytes().second

            if(!(command is HeartBeatRequest)) {
                logger.debug { "send message: ${message.toHumanRead()}" }
            }
            try {
                if(!socket.isClosed) {
                    //logger.debug { "sendMessage : ${data.toHumanRead()}" }
                    socket.send(Datagram(ByteReadPacket(message), socket.remoteAddress))
                }else {
                    logger.warn { "can not send message to UDP Server:$targetAddress"}
                }
            }catch (e:Exception) {
                e.printStackTrace()
            }

        }
    }

    fun close(){
        socket.close()
    }
}
