package com.mjitech.third.down.entity

import com.mjitech.api.SkuAPI
import com.mjitech.api.model.OrderBatch
import com.mjitech.api.model.SKU
import com.mjitech.third.down.protocol.DownEncoder
import kotlinx.serialization.Serializable
import mu.KotlinLogging

@Serializable
data class OutGood(
    val shelfNumber: Byte,
    val x: Int,
    val y: Int,
    val x1: Int,
    val y1: Int,
    val number: Byte, // 数量
    val action: Byte
)

@Serializable
data class OutGoods(
    val outGoods: List<OutGood>
) : CommandRequest {
    override fun toBytes(): Pair<Int,ByteArray> {
        return serializer().toBytes(this, type)
    }

    companion object : CommandDesc {
        override val type: Byte = 0x01.toByte()

        fun fromOrderBatch(batch: OrderBatch): OutGoods {
            val outGoodList = batch.skus.map { sku ->
                convertSKUToOutGood(sku)
            }
            return OutGoods(outGoodList)
        }
        private fun convertSKUToOutGood(sku:SKU):OutGood {
            return OutGood(
                    sku.shelfNumber.toByte(),
                    sku.x,
                    sku.y,
                    sku.x1,
                    sku.y1,
                    1.toByte(),
                    1.toByte())
        }

        fun fromSKU(sku:SKU): OutGoods {
            return OutGoods(listOf(convertSKUToOutGood(sku)))
        }
    }
}
