package com.mjitech.third.down.entity.server

import com.mjitech.third.down.entity.CommandDesc
import com.mjitech.third.down.entity.CommandResponse
import com.mjitech.third.down.entity.server.HeartBeatResponse.Companion.Safe
import com.mjitech.third.down.entity.toBytes
import kotlinx.serialization.Serializable
import kotlin.experimental.or

// 3.10. ~心跳B（该协议既针对下单屏还针对维护屏
@Serializable
data class ACKResponse(
    val originalFrameNumber: Int,
    val typ: Byte = 0x09.toByte(),
    val leftHorseBlock: Byte, // 左踏台
    val rightHorseBlock: Byte,
    val replenishmentScreen: Byte, // 补货屏
    val leftDoor: Byte, // 左柜门
    val rightDoor: Byte,
    val electronDoor: Byte, // 电柜门
    val sealPort: Byte, // 密封口
    val shipmentWarehouse: Byte, // 出货仓
    val innerDoor: Byte // 货仓内部关门
) : CommandResponse {
    override fun toBytes(): Pair<Int, ByteArray> {
        return serializer().toBytes(this, type)
    }

    // TODO: 这个和心跳逻辑一致
    fun isSafe(): Boolean {
        val result =
            (leftHorseBlock or rightHorseBlock or replenishmentScreen).toInt()
        return result == 1.toByte().toInt()
    }

    companion object : CommandDesc {
        override val type: Byte = 0x0a.toByte()
    }
}
