package com.mjitech.third.down.mock

import com.mjitech.third.down.entity.CommandResponse
import com.mjitech.third.down.mock.entity.HeartBeatState
import io.ktor.network.selector.ActorSelectorManager
import io.ktor.network.sockets.Datagram
import io.ktor.network.sockets.aSocket
import io.ktor.utils.io.core.ByteReadPacket
import io.ktor.utils.io.core.readBytes
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import mu.KLogger
import mu.KotlinLogging
import java.net.InetSocketAddress
import java.net.SocketAddress
import kotlin.coroutines.CoroutineContext

class MockUDPDownServer(
    port: Int,
    override val coroutineContext: CoroutineContext

) : MockDownServer {
    override val logger: KLogger = KotlinLogging.logger { }
    private var address: SocketAddress? = null
    val server = aSocket(ActorSelectorManager(Dispatchers.IO)).udp().bind(
        InetSocketAddress("0.0.0.0", port)
    )
    private  var heartBeatState = HeartBeatState()



    init {
        launch {
            logger.info("mock upd server: $port init success")
            //server.send(DatagramPacket())
            while (true) {
                try {
                    for (data in server.incoming) {
                        val heartBeatResponse = heartBeatState.getHeartBeatState()
                        address = data.address
                        this@MockUDPDownServer.listen(data.packet.readBytes(),heartBeatResponse)
                    }
                } finally {
                    break
                }
            }
        }
    }


    override fun sendCommand(command: CommandResponse) {
        launch {
            address?.let {
                server.send(Datagram(ByteReadPacket(command.toBytes().second), it))
            }
        }
    }

    fun close(){
        server.close()
    }
}
