package com.mjitech.third.down.protocol

import kotlinx.serialization.CompositeEncoder
import kotlinx.serialization.Encoder
import kotlinx.serialization.KSerializer
import kotlinx.serialization.PrimitiveKind
import kotlinx.serialization.SerialDescriptor
import kotlinx.serialization.SerializationStrategy
import kotlinx.serialization.StructureKind
import kotlinx.serialization.elementDescriptors
import kotlinx.serialization.modules.EmptyModule
import kotlinx.serialization.modules.SerialModule
import java.util.Arrays

class DownEncoder(private val type: Byte) : Encoder, CompositeEncoder {
    override val context: SerialModule = EmptyModule
    private var array: ByteArray = byteArrayOf()
    private var frameNumber: Int = 0
    private var frameBodyLength: Int = 0
    private var pos = 0

    fun getData(): ByteArray {
        return array
    }

    fun getFrameNumber(): Int {
        return frameNumber
    }

    override fun beginStructure(
        descriptor: SerialDescriptor,
        vararg typeSerializers: KSerializer<*>
    ): CompositeEncoder {
        val length = descriptor.elementDescriptors().map {
            when (it.kind) {
                PrimitiveKind.SHORT -> 2
                PrimitiveKind.INT -> 4
                PrimitiveKind.BYTE -> 1
                StructureKind.LIST -> 0
                StructureKind.CLASS -> 0
                else ->
                    throw IllegalArgumentException("does not support ${it.kind}")
            }
        }.sum()
        if (length > 0) {
            array = Arrays.copyOf(array, array.size + length)
        }

        return this
    }

    override fun encodeBoolean(value: Boolean) {
        TODO("Not yet")
    }

    override fun encodeByte(value: Byte) {
        TODO("Not yet implemented")
    }

    override fun encodeChar(value: Char) {
        TODO("Not yet implemented")
    }

    override fun encodeDouble(value: Double) {
        TODO("Not yet implemented")
    }

    override fun encodeEnum(enumDescriptor: SerialDescriptor, index: Int) {
        TODO("Not yet implemented")
    }

    override fun encodeFloat(value: Float) {
        TODO("Not yet implemented")
    }

    override fun encodeInt(value: Int) {
        TODO("Not yet implemented")
    }

    override fun encodeLong(value: Long) {
        TODO("Not yet implemented")
    }

    override fun encodeNull() {
        TODO("Not yet implemented")
    }

    override fun encodeShort(value: Short) {
        TODO("Not yet implemented")
    }

    override fun encodeString(value: String) {
        TODO("Not yet implemented")
    }

    override fun encodeUnit() {
        TODO("Not yet implemented")
    }

    // TODO: do nothing
    override fun encodeBooleanElement(descriptor: SerialDescriptor, index: Int, value: Boolean) {
        TODO("Not yet implemented")
    }

    override fun encodeByteElement(descriptor: SerialDescriptor, index: Int, value: Byte) {
        pos = value.inBytes(array, pos)
    }

    override fun encodeCharElement(descriptor: SerialDescriptor, index: Int, value: Char) {
        TODO("Not yet implemented")
    }

    override fun encodeDoubleElement(descriptor: SerialDescriptor, index: Int, value: Double) {
        TODO("Not yet implemented")
    }

    override fun encodeFloatElement(descriptor: SerialDescriptor, index: Int, value: Float) {
        TODO("Not yet implemented")
    }

    override fun encodeIntElement(descriptor: SerialDescriptor, index: Int, value: Int) {
        pos = value.inBytes(array, pos)
    }

    override fun encodeLongElement(descriptor: SerialDescriptor, index: Int, value: Long) {
        TODO("Not yet implemented")
    }

    override fun <T : Any> encodeNullableSerializableElement(
        descriptor: SerialDescriptor,
        index: Int,
        serializer: SerializationStrategy<T>,
        value: T?
    ) {
        TODO("Not yet implemented")
    }

    override fun <T> encodeSerializableElement(
        descriptor: SerialDescriptor,
        index: Int,
        serializer: SerializationStrategy<T>,
        value: T
    ) {
        serializer.serialize(this, value)
    }

    override fun encodeShortElement(descriptor: SerialDescriptor, index: Int, value: Short) {
        pos = value.inBytes(array, pos)
    }

    override fun encodeStringElement(descriptor: SerialDescriptor, index: Int, value: String) {
        TODO("Not yet implemented")
    }

    override fun encodeUnitElement(descriptor: SerialDescriptor, index: Int) {
        TODO("Not yet implemented")
    }

    override fun endStructure(descriptor: SerialDescriptor) {
        val headArraySize = FrameBodyPosition
        val tailArraySize = 1 + 2
        val frameLength = headArraySize + array.size + tailArraySize

        val headArray = ByteArray(headArraySize)
        var p = 0
        p = 0x7D.toByte().inBytes(headArray, p)// frame header
        p = 0x7E.toByte().inBytes(headArray, p)
        p = frameLength.toByte().inBytes(headArray, p) //frame length
        frameNumber = generateFrameNumber()
        p = frameNumber.inBytes(headArray, p)
        type.inBytes(headArray, p)
        array = headArray + array
        val sum = checkSum(array, 2, array.size - 1)
        array += byteArrayOf(sum, 0x0D.toByte(), 0x0A.toByte())
    }
}
