package com.mjitech.third.down.protocol

import com.fazecast.jSerialComm.SerialPort
import com.fazecast.jSerialComm.SerialPortEvent
import com.fazecast.jSerialComm.SerialPortMessageListener

// Frame 格式：
// frame header(2) + frame length(1) + frame number(4) + frame type(1) + frame data(length) + frame checksum(1) + frame end(2)

fun connectionOptions(port: SerialPort) {
    port.baudRate = 19200
}

@Deprecated("只能用于 downClient")
fun createMessageListener(func: (SerialPortEvent) -> Unit): SerialPortMessageListener {
    return MessageListener(func)
}

fun Int.inBytes(arr: ByteArray, startPoint: Int): Int {
    arr[startPoint + 3] = this.toByte()
    arr[startPoint + 2] = (this shr 8).toByte()
    arr[startPoint + 1] = (this shr 16).toByte()
    arr[startPoint] = (this shr 24).toByte()
    return startPoint + 4
}

fun Short.inBytes(arr: ByteArray, startPoint: Int): Int {
    arr[startPoint + 1] = this.toByte()
    // Attention: Fuck Kotlin, do not support Short shr
    arr[startPoint] = (this.toInt() shr 8).toByte()
    return startPoint + 2
}

fun Byte.inBytes(arr: ByteArray, startPoint: Int): Int {
    arr[startPoint] = this
    return startPoint + 1
}

fun ByteArray.getInt(startPoint: Int): Int {
    return (this[startPoint].toInt() shl 24) or
        (this[startPoint + 1].toInt() and 0xff shl 16) or
        (this[startPoint + 2].toInt() and 0xff shl 8) or
        (this[startPoint + 3].toInt() and 0xff)
}

fun ByteArray.getShort(startPoint: Int): Short {
    return (
        (this[startPoint].toInt() shl 8) or
            (this[startPoint + 1].toInt() and 0xff)
        ).toShort()
}
/*
private fun intToBytes(v: Int, length: Int): List<Int> {
    return when (v) {
        4 -> {
            listOf(
                (v shr 24 and 0xff),
                v shr 16 and 0xff,
                v shr 8 and 0xff,
                v and 0xff
            )
        }
        2 -> {
            listOf(
                v shr 8 and 0xff,
                v and 0xff
            )
        }
        else -> listOf(v)
    }
}
 */

fun generateFrameNumber(): Int {
    return (System.currentTimeMillis() / 1000).toInt()
}

fun checkSum(array: ByteArray, beginPos: Int, length: Int): Byte {
    return (
        0xff - (
            beginPos.rangeTo(length)
                .fold(0) { acc, index -> acc + (array[index].toInt() and 0xff) } and 0xff
            )
        ).toByte()
}

/*
fun commandToFrame(v: Command): Pair<Int, ByteArray> {


    val frameLength = 2 + 1 + 4 + 1 + v.desc.length + 1 + 2
    val frameNumber = generateFrameNumber()
    val array =
        ByteArray(frameLength)
    var pos = 0
    pos = 0x7D.toByte().inBytes(array, pos) //frame header
    pos = 0x7E.toByte().inBytes(array, pos)
    val beginDataPos = pos
    pos = frameLength.toByte().inBytes(array, pos) //frame length
    pos = frameNumber.inBytes(array, pos) // frame  number
    pos = v.desc.type.inBytes(array, pos) //frame type
    pos = v.inBytes(array, pos) // frame data
    pos = checkSum(array, beginDataPos, v.desc.length).inBytes(array, pos) // frame check sum
    pos = 0x0D.toByte().inBytes(array, pos) // frame end
    pos = 0x0A.toByte().inBytes(array, pos)
    assert(pos == array.size)
    return frameNumber to array
}

 */

val FrameBodyPosition: Int = 2 + 1 + 4 + 1

fun ByteArray.getFrameNumber(): Int {
    return this.getInt(3)
}

fun ByteArray.getFrameType(): Byte {
    return this.get(7)
}

fun ByteArray.toHumanRead(): List<String> {
    return this.map { it.toHumanRead() }
}
fun Byte.toHumanRead():String {
    return Integer.toHexString(this.toInt() and 0xff)
}
/*
fun dataToFrame(data: Array<Int>, dataLength: Array<Int>): Pair<List<Int>, List<Int>> {
    assert(data.size == dataLength.size)
    val frameNumberBytes = intToBytes((System.currentTimeMillis() / 1000).toInt(), 4)

    val result = data.withIndex().fold(frameNumberBytes.toMutableList(), { result, item ->
        result.addAll(intToBytes(item.value, item.index))
        result
    })
    //frame length
    result.add(0, result.size + 6)

    val sumCheck = 0xff - (result.reduce { pre, current -> pre + current } and 0xff)

    // frame head
    result.addAll(0, listOf(125, 126))
    //frame sum check
    result.add(sumCheck)
    // frame tail
    result.addAll(listOf(13, 10))

    return Pair(frameNumberBytes, result.toList())
}
 */

private class MessageListener(private val func: (SerialPortEvent) -> Unit) : SerialPortMessageListener {
    override fun delimiterIndicatesEndOfMessage(): Boolean {
        return true
    }

    override fun serialEvent(event: SerialPortEvent) {
        func(event)
    }

    override fun getMessageDelimiter(): ByteArray {
        return byteArrayOf(13, 10)
        // return "\r\n".toByteArray()
    }

    override fun getListeningEvents(): Int {
        return SerialPort.LISTENING_EVENT_DATA_RECEIVED
    }
}
