package com.mjitech.third.scancode

import com.fazecast.jSerialComm.SerialPort
import com.fazecast.jSerialComm.SerialPortEvent
import com.fazecast.jSerialComm.SerialPortMessageListener
import mu.KotlinLogging

class ScanCodeClient(
    portDescriptor: String,
    val scanCodeChannel: ScanCodeChannel
) : SerialPortMessageListener {
    private val logger = KotlinLogging.logger {}
    private val port: SerialPort = SerialPort.getCommPort(portDescriptor)

    init {
        connectionOptions(port)
        port.addDataListener(this)
        val result = port.openPort()
        if(result) {
            logger.info("Scan Code init...port:$portDescriptor success")
        }else {
            logger.error("Scan Code init...port:$portDescriptor failure")
        }
    }

    override fun delimiterIndicatesEndOfMessage(): Boolean {
        return true
    }

    override fun serialEvent(event: SerialPortEvent) {
        // https://www.wechat.com?redirect=xxx=abc=take_good_13213
        val str = String(event.receivedData).split(MessagePrefix)[1].split(Delimiter)[0]
        logger.info { "receive message: $str" }
        scanCodeChannel.send(str)
    }

    override fun getMessageDelimiter(): ByteArray {
        return Delimiter.toByteArray()
    }

    override fun getListeningEvents(): Int {
        return SerialPort.LISTENING_EVENT_DATA_RECEIVED
    }

    fun close(){
        port.removeDataListener()
        port.closePort()
    }
}
