package com.timzaak.jackson

import com.fasterxml.jackson.annotation.JsonInclude
import com.fasterxml.jackson.databind.DeserializationFeature
import com.fasterxml.jackson.databind.ObjectMapper
import com.fasterxml.jackson.databind.SerializationFeature
import com.fasterxml.jackson.databind.node.ObjectNode
import com.fasterxml.jackson.datatype.joda.JodaModule
import com.fasterxml.jackson.module.kotlin.jacksonObjectMapper
import io.ktor.features.ContentNegotiation
import io.ktor.http.ContentType
import io.ktor.jackson.JacksonConverter

// Jackson 在 Ktor 下的配置
object KtorJacksonWrapper {
    // need private..
    val jacksonMapper: ObjectMapper = jacksonObjectMapper()
        .setSerializationInclusion(JsonInclude.Include.NON_NULL)
        .disable(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS)
        .registerModules(
            JodaModule()
        )
        .configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false)
        .configure(DeserializationFeature.FAIL_ON_NULL_FOR_PRIMITIVES, true)

    fun ContentNegotiation.Configuration.jackson(contentType: ContentType = ContentType.Application.Json) {
        val converter = JacksonConverter(jacksonMapper)
        register(contentType, converter)
    }

    fun toObjectNodeUnsafe(txt:String):ObjectNode {
        return jacksonMapper.readTree(txt) as ObjectNode
    }
}
