package com.timzaak.ucloud.mqtt

import com.fasterxml.jackson.databind.ObjectMapper
import io.ktor.client.HttpClient
import io.ktor.client.request.header
import io.ktor.client.request.request
import io.ktor.client.statement.HttpResponse
import io.ktor.client.statement.readText
import io.ktor.http.ContentType
import io.ktor.http.HttpMethod
import io.ktor.http.contentType
import org.apache.commons.codec.digest.HmacUtils
import org.apache.commons.codec.digest.Md5Crypt
import java.io.File

class UIotFileClient(
    private val httpClient: HttpClient,
    private val mqttConfig: MqttConfig,
    private val objectMapper: ObjectMapper
) {

    suspend fun postFile(file: File, name: String, contentType: String = "plain/text") {
        val data = file.readBytes()
        val md5 = Md5Crypt.md5Crypt(data)
        val result = httpClient.request<HttpResponse>("https://file-cn-sh2.iot.ucloud.cn") {
            method = HttpMethod.Post
            val b = addSecretBody(name, data, md5, contentType)
            contentType(ContentType.Application.Json)
            header("Authorization", sign(b))
            body = b
        }
        val json = objectMapper.readTree(result.readText(Charsets.UTF_8))
        if (json["RetCode"].asInt() == 0) {
            val url = json["URL"].asText()
            val auth = json["Authorization"].asText()
            httpClient.request<HttpResponse>(url) {
                method = HttpMethod.Put
                header("Authorization", auth)
                header("Content-type", contentType)
                header("Content-MD5", md5)
            }
        }
    }

    private fun sign(data: String): String {
        return String(HmacUtils.getHmacSha256(mqttConfig.devicePassword.toByteArray()).doFinal(data.toByteArray()))
    }

    private fun addSecretBody(name: String, data: ByteArray, md5: String, contentType: String): String {
        return objectMapper.writeValueAsString(
            linkedMapOf(
                "ProductSN" to mqttConfig.productSN,
                "DeviceSN" to mqttConfig.deviceSN,
                "FileName" to name,
                "FileSize" to data.size,
                "MD5" to md5,
                "Content-Type" to contentType
            )
        )
    }
}
