package com.timzaak.ucloud.mqtt.entity

import com.fasterxml.jackson.annotation.JsonProperty
import com.fasterxml.jackson.databind.JsonNode
import com.fasterxml.jackson.databind.node.JsonNodeFactory
import com.fasterxml.jackson.databind.node.ObjectNode
import com.mjitech.service.mqtt.command.getIdentifier
import com.mjitech.service.mqtt.command.getRequestID
import org.joda.time.DateTime
import java.util.UUID

interface Request {
    val RequestID: String
}

data class PostProperty(
    @get:JsonProperty("Property")
    val Property: Map<String, PropertyValue<*>>,
    @get:JsonProperty("RequestID")
    override val RequestID: String = UUID.randomUUID().toString()
) : Request

data class PropertyValue<T>(
    @get:JsonProperty("Value")
    val Value: T,
    @get:JsonProperty("Time")
    val Time: Long = DateTime.now().millis
)

data class PostEvent(
    @get:JsonProperty("Identifier")
// @param:JsonProperty("Identifier")
    val Identifier: String,
    @get:JsonProperty("Output")
    val Output: ObjectNode,
    @get:JsonProperty("Time")
    val Time: Long = DateTime.now().millis,
    @get:JsonProperty("RequestID")
    override val RequestID: String = UUID.randomUUID().toString()
) : Request

data class ResponseCommand(
    @get:JsonProperty("RequestID")
    override val RequestID: String,
    @get:JsonProperty("Identifier")
    val Identifier: String,
    @get:JsonProperty("Output")
    val Output: ObjectNode,
    @get:JsonProperty("RetCode")
    val RetCode: Int = 0
) : Request {
    companion object {
        fun baseResp(json: JsonNode): ResponseCommand {
            return ResponseCommand(
                json.getRequestID(),
                json.getIdentifier(),
                JsonNodeFactory.instance.objectNode()
            )
        }
    }
}
