package com.timzaak.ws

import io.ktor.http.cio.websocket.Frame
import io.ktor.http.cio.websocket.WebSocketSession
import io.ktor.http.cio.websocket.close
import kotlinx.coroutines.isActive
import java.util.concurrent.ConcurrentHashMap

class ConnectionManager {
    private val map = ConcurrentHashMap<String, WebSocketSession>()

    suspend fun memberJoin(sessionId: String, socket: WebSocketSession) {
        map.put(sessionId, socket)?.close()
    }

    suspend fun memberLeft(sessionId: String) {
        map.remove(sessionId)?.let {
            if (it.isActive) {
                it.close()
            }
        }
    }

    suspend fun broadcastAll(frame: Frame) {
        for ((sessionId, socket) in map.toMap()) {
            if (socket.isActive) {
                socket.outgoing.send(frame)
            } else {
                memberLeft(sessionId)
            }
        }
    }
}
