package di

import com.mjitech.service.DownClientService
import com.mjitech.service.ShopService
import com.mjitech.service.mqtt.CloudMqttService
import com.mjitech.service.state.StateManager
import io.ktor.application.Application
import kotlinx.coroutines.channels.consumeEach
import kotlinx.coroutines.launch
import mu.KotlinLogging
import org.koin.ktor.ext.inject

private val logger = KotlinLogging.logger { }
fun Application.lazyInitService() {
    val shopService: ShopService by inject()
    val cloudMqttService: CloudMqttService by inject()
    val downClientService: DownClientService by inject()
    // val repairService: RepairService by inject()
    val stateManager: StateManager by inject()

    listOf(
        shopService,
        downClientService,
//        repairService,
        cloudMqttService
    ).forEach { service ->
        launch {
            stateManager.subscribe().consumeEach {
                try {
                    service.consume(it)
                } catch (exception: Throwable) {
                    logger.warn(exception) { "service: $service consume $it failure" }
                }
            }
        }
    }
}
