package di

import com.mjitech.api.OrderAPI
import com.mjitech.api.SkuAPI
import com.mjitech.service.DownClientService
import com.mjitech.service.RepairService
import com.mjitech.service.ShopService
import com.mjitech.service.mqtt.CloudMqttService
import com.mjitech.service.state.StateManager
import com.mjitech.third.down.DownClient
import com.mjitech.third.down.DownClientChannel
import com.mjitech.third.down.DownUDPClient
import com.mjitech.third.down.mock.MockUDPDownServer
import com.mjitech.third.scancode.ScanCodeChannel
import com.mjitech.third.scancode.ScanCodeClient
import com.mjitech.third.scancode.mock.MockScanCodeServer
import com.timzaak.coroutines.channel.BaseScopeBroadcastChannel
import com.timzaak.coroutines.channel.BaseScopeChannel
import com.timzaak.docker.DockerClient
import com.timzaak.docker.DockerClientImp
import com.timzaak.docker.mock.MockDockerClient
import com.timzaak.jackson.KtorJacksonWrapper
import com.timzaak.ucloud.mqtt.MqttConfig
import com.timzaak.ucloud.mqtt.MqttMessageChannel
import com.timzaak.ucloud.mqtt.MqttStateChannel
import com.timzaak.ucloud.mqtt.UIotFileClient
import com.timzaak.ucloud.mqtt.UMqttClient
import com.typesafe.config.Config
import config.isEnableMockDownServer
import config.isEnableMockScanCodeServer
import config.isMockDockerClient
import org.koin.core.module.Module
import org.koin.dsl.module
import java.net.InetSocketAddress

fun thirdDI(config: Config): Module {
    return module {
        single(createdAtStart = true) {
            DownClientChannel(BaseScopeBroadcastChannel(5, get()))
        }
        single(createdAtStart = true) {
            DownUDPClient(
                InetSocketAddress(
                    config.getString("third.downClient.downServer.ip"),
                    config.getInt("third.downClient.downServer.port")
                ),
                get(),
                get()
            ) as DownClient
        }

        single(createdAtStart = true) {
            MqttMessageChannel(BaseScopeBroadcastChannel(1, get()))
        }
        single(createdAtStart = true) {
            MqttStateChannel(BaseScopeBroadcastChannel(1, get()))
        }
        single {
            MqttConfig.fromConfig(config.getConfig("api.mqtt"))
        }
        single {
            KtorJacksonWrapper.jacksonMapper
        }
        single {
        }
        single(createdAtStart = true) {
            UMqttClient(get(), get(), get(), get(), get())
        }
        single {
            UIotFileClient(get(), get(), get())
        }

        single(createdAtStart = true) {
            ScanCodeChannel(BaseScopeChannel(get()))
        }
        single(createdAtStart = true) {
            ScanCodeClient(config.getString("third.scanCodeClient.port"), get())
        }
    }
}

fun thirdMockDI(config: Config): Module {
    return module {
        if (config.isEnableMockDownServer()) {
            single(createdAtStart = true) {
                MockUDPDownServer(
                    config.getInt("third.downClient.downServer.port"),
                    get()
                )
            }
        }
        if (config.isEnableMockScanCodeServer()) {
            single(createdAtStart = true) {
                MockScanCodeServer(config.getString("mock.third.scanCodeServer.port"))
            }
        }
    }
}

fun serviceDI(config:Config): Module {
    return module {
        single {
            StateManager(get())
        }

        single {
            ShopService(get(), get(), get(), get(), get(), get(), get())
        }

        single {
            (if(config.isMockDockerClient()){
                MockDockerClient()
            }else {
                DockerClientImp()
            }) as DockerClient
        }

        single {
            CloudMqttService(get(), get(), get(), get(), get(), get(), get(), get(), get())
        }

        single {
            DownClientService(get(), get(), get())
        }

        single {
            RepairService(get(), get())
        }
    }
}

fun apiDI(): Module {
    return module {
        single {
            OrderAPI(get())
        }
        single {
            SkuAPI(get())
        }
    }
}

