package routes

import com.fasterxml.jackson.databind.node.ArrayNode
import com.fasterxml.jackson.databind.node.ObjectNode
import com.mjitech.api.BaseAPIImpl
import com.timzaak.jackson.KtorJacksonWrapper
import io.ktor.application.call
import io.ktor.client.statement.readText
import io.ktor.http.Parameters
import io.ktor.http.isSuccess
import io.ktor.http.parametersOf
import io.ktor.http.plus
import io.ktor.request.queryString
import io.ktor.request.receiveOrNull
import io.ktor.response.respond
import io.ktor.routing.Route
import io.ktor.routing.post
import io.ktor.routing.route
import mu.KotlinLogging

private val logger = KotlinLogging.logger { }

fun Route.redirectToServer(api: BaseAPIImpl) {

    redirectMap.map { (remote, local) ->
        route(local) {
            post {

                val queryString = call.request.queryString()
                val url = if (queryString.isEmpty()) {
                    remote
                } else {
                    "$remote?${call.request.queryString()}"
                }
                logger.debug { "proxy $local to $url" }
                val text: String? = call.receiveOrNull()

                val obj = if (text.isNullOrEmpty()) {
                    Parameters.Empty
                } else {
                    (KtorJacksonWrapper.jacksonMapper.readTree(text) as ObjectNode).fields().asSequence()
                        .fold(Parameters.Empty) { acc, i ->
                            when (val v = i.value) {
                                is ArrayNode -> {
                                    acc + v.fold(Parameters.Empty) { acc1, i1 ->
                                        acc1 + parametersOf(i.key, i1.asText())
                                    }
                                }
                                else -> {
                                    acc + parametersOf(i.key, i.value.asText())
                                }
                            }
                        }
                }

                val response = api.post(url, obj)
                if (response.status.isSuccess()) {
                    call.respond(response.readText())
                } else {
                    val data = response.readText()
                    logger.info { data }
                    call.respond("{\"is_success\":false}")
                }
            }
        }
    }
}

private val redirectMap = mapOf(
    "/machine_api/get_mainpage_data.action" to "/maxbox_pc/local_api/get_mainpage_data.action",
    "/machine_api/add_sku_to_cart.action" to "/maxbox_pc/local_api/add_sku_to_cart.action",
    "/machine_api/remove_sku_from_cart.action" to "/maxbox_pc/local_api/remove_sku_from_cart.action",
    "/machine_api/get_cart.action" to "/maxbox_pc/local_api/get_cart.action",
    "/machine_api/clear_cart.action" to "/maxbox_pc/local_api/clear_cart.action",
    "/machine_api/submit_cart.action" to "/maxbox_pc/local_api/submit_cart.action",
    "/machine_api/submit_cart_new.action" to "/maxbox_pc/local_api/submit_cart_new.action",
    "/machine_api/get_pay_url.action" to "/maxbox_pc/local_api/request_pay.action",
    "/machine_api/get_order_detail.action" to "/maxbox_pc/local_api/get_order_detail.action",
    "/machine_api/get_sku_detail.action" to "/maxbox_pc/local_api/sku_detail.action",
    "/machine_api/get_marketing_data.action" to "/maxbox_pc/local_api/get_marketing_data.action",
    "/machine_api/get_channel_detail.action" to "/maxbox_pc/local_api/get_channel_detail.action",
    "/machine_api/get_campaign_detail.action" to "/maxbox_pc/local_api/get_campaign_detail.action",
    "/machine_api/get_brand_detail.action" to "/maxbox_pc/local_api/brand_detail.action",
    "/machine_api/exchange_sku.action" to "/maxbox_pc/local_api/get_missing_payurl",
    "/machine_api/media_screen_data.action" to "/maxbox_pc/local_api/media_screen_data.action",
    "/machine_api/media_screen_weather_data.action" to "/maxbox_pc/local_api/media_screen_weather_data.action",
    "/machine_api/get_promotionHgSku_toCart.action" to "/maxbox_pc/machine_api/get_promotionHgSku_toCart.action",
    "/machine_api/get_store_info.action" to "/maxbox_pc/machine_api/get_store_info.action",
    "/machine_api/user_cancel_order.action" to "/maxbox_pc/machine_api/user_cancel_order.action",
    "/machine_api/media_screen_data_new.action" to "/maxbox_pc/local_api/media_screen_data_new.action",
    "/machine_api/resetting_order.action" to "/maxbox_pc/local_api/resetting_order.action",
    "/machine_api/refund_order_machine.action" to "/maxbox_pc/local_api/refund_order_machine.action",
    "/machine_api/get_order_detail_by_takingnumber.action" to "/maxbox_pc/local_api/get_order_detail_by_takingnumber.action",
    "/machine_api/fail_get_sku.action" to "/maxbox_pc/local_api/fail_get_sku.action",
    "/machine_api/set_outbatchsku_status.action" to "/maxbox_pc/local_api/set_outbatchsku_status.action",
    "/machine_api/update_order_status.action" to "/maxbox_pc/local_api/update_order_status.action",
    "/machine_api/get_exchanged_sku.action" to "/maxbox_pc/local_api/get_exchanged_sku.action",
    "/machine_api/refund_failed_get_sku.action" to "/maxbox_pc/local_api/refund_failed_get_sku.action",
    "/machine_api/refund_failed_get_sku.action" to "/maxbox_pc/local_api/refund_failed_get_sku.action",
    "/machine_api/upload_error.action" to "/maxbox_pc/local_api/upload_error.action",
    "/machine_api/upload_error_detail.action" to "/maxbox_pc/local_api/upload_error_detail.action",
    "/machine_api/get_all_inventories.action" to "/maxbox_pc/local_api/get_all_inventories.action"
)
