package routes

import com.fasterxml.jackson.databind.JsonNode
import com.fasterxml.jackson.databind.ObjectMapper
import com.fasterxml.jackson.module.kotlin.convertValue
import com.mjitech.api.model.CurrentTaking
import com.mjitech.compatible.statusIdAdapt
import com.mjitech.service.state.StateManager
import com.mjitech.service.state.entity.State
import com.timzaak.jackson.jsObject
import config.AppConfig
import io.ktor.application.call
import io.ktor.http.content.resolveResource
import io.ktor.request.receive
import io.ktor.response.respond
import io.ktor.routing.Route
import io.ktor.routing.get
import io.ktor.routing.post
import io.ktor.routing.route
import org.koin.ktor.ext.inject

fun Route.upperRouters(appConfig: AppConfig) {
    val stateManager: StateManager by inject()
    val objectMapper: ObjectMapper by inject()
    // 服务状态
    route("/machinestatus") {
        post {
            val body = call.receive<JsonNode>()
            val screenId = body["screenId"].asText().toInt()
            val cacheData = stateManager.getWebResponse(screenId)

            val state = stateManager.state
            if (screenId == 7) {
                when (state) {
                    is State.DoBusiness.TakingGood -> {
                        if (state.failTaking != null && state.errorStep != -1) {
                            println(cacheData)
                            val order = state.order
                            val sku = order.getSKUByStep(state.batchStep,state.errorStep)
                            call.respond(
                                jsObject {
                                    "status" /= state.statusIdAdapt(cacheData)
                                    "currentTaking" /= objectMapper.convertValue<JsonNode>(
                                        CurrentTaking(
                                            sku!!.skuId,
                                            order.orderNumber,
                                            sku,
                                            sku.outBatchId
                                        )
                                    )
                                    "failTaking" /= objectMapper.convertValue<JsonNode>(state.failTaking)
                                }.asTree()
                            )
                        } else {
                            call.respond(
                                jsObject {
                                    "status" /= state.statusIdAdapt(cacheData)
                                    "currentTaking" /= ""
                                    "failTaking" /= ""
                                }.asTree()
                            )
                        }
                    }
                    else -> {
                        call.respond(
                            jsObject {
                                "status" /= state.statusIdAdapt(cacheData)
                                "currentTaking" /= ""
                                "failTaking" /= ""
                            }.asTree()
                        )
                    }
                }
            } else {
                call.respond(
                    jsObject {
                        "status" /= state.statusIdAdapt(cacheData)
                    }.asTree()
                )
            }
        }
// 27,65 只用了 status 字段.
// 7 status, currentTaking,failTaking
//        status: status.status,
//        meta: status.meta||'', 27,65
//        currentBatch: takingCache.getCurrentBatch(), 27 ,65
//        currentBatchIndex: takingCache.getCurrentBatchIndex(), 27,65
//        totalBatchCounts: takingCache.getTotalBatchCounts(), 27,65
//        currentTaking: takingCache.current()||'', 27,65
//        failTaking: missingCache.getCurrent()||'', 27,65
//        isMachineWorking: machineStatus.isWorking()//27,65
    }

    route("/maxbox_pc/get_current_domain.action") {
        post {
            val domain = appConfig.remoteServer.replace("web", "")
            val b = true
            call.respond(
                jsObject {
                    "is_succ" /= b
                    "domain" /= domain
                }.asTree()
            )
        }
    }

    route("/getmyid") {
        get {
            call.respond(
                jsObject {
                    "storeId" /= appConfig.storeId
                }.asTree()
            )
        }
    }

    route("/userinteract") {
        post {
            call.respond(
                jsObject {}.asTree()
            )
        }
    }
    route("/errortrack") {
        post {
            call.respond(
                jsObject {}.asTree()
            )
        }
    }
    route("/track") {
        post {
            call.respond(
                jsObject {}.asTree()
            )
        }
    }
    route("/defaultMedia/back.mp3") {
        get {
            call.respond(call.resolveResource("back.mp3", "defaultMedia")!!)
        }
    }
    route("/defaultMedia/opendoor.mp3") {
        get {
            call.respond(call.resolveResource("opendoor.mp3", "defaultMedia")!!)
        }
    }
    route("/defaultMedia/closedoor.mp3") {
        get {
            call.respond(call.resolveResource("closedoor.mp3", "defaultMedia")!!)
        }
    }
    route("/defaultMedia/qrcode.mp3") {
        get {
            call.respond(call.resolveResource("qrcode.mp3", "defaultMedia")!!)
        }
    }
    route("/defaultMedia/paysucc.mp3") {
        get {
            call.respond(call.resolveResource("paysucc.mp3", "defaultMedia")!!)
        }
    }
}
