package routes.pos

import com.fasterxml.jackson.databind.node.ObjectNode
import com.mjitech.api.CardPayOrder
import com.mjitech.api.OrderAPI
import com.timzaak.jackson.jsObject

object PosResult{
    //根据pos返回码返回结果
    suspend fun getResultByCode(res:ObjectNode,orderAPI: OrderAPI,orderNumber:String):String{
        when (res.get("rejcode").asText()) {
            PosStatus.succ -> {
                val cardNumber = res.get("cardno").asText()
                val traceNumber = res.get("trans_ticket_no").asText()
                orderAPI.updateCardPay(orderNumber, traceNumber, cardNumber)
                return resultContent(
                        true,
                        "",
                        1,
                        orderNumber,
                        true,
                        "",
                        1
                )
            }

            PosStatus.rePay -> {
                return resultContent(
                        false,
                        "订单已交易过",
                        -6,
                        orderNumber,
                        false,
                        "订单已交易过",
                        -6
                )
            }
            PosStatus.cancel -> {
                return resultContent(
                        false,
                        "用户取消刷卡",
                        -2,
                        orderNumber,
                        false,
                        "用户取消刷卡",
                        -2
                )
            }
            PosStatus.overtime -> {
                return resultContent(
                        false,
                        "用户刷卡超时",
                        -2,
                        orderNumber,
                        false,
                        "用户刷卡超时",
                        -2
                )
            }
            PosStatus.pwdError -> {
                return resultContent(
                        true,
                        "密码错误",
                        -4,
                        orderNumber,
                        true,
                        "密码错误",
                        -4
                )
            }
            PosStatus.wait -> {
                return resultContent(
                        true,
                        "等待",
                        -5,
                        orderNumber,
                        true,
                        "等待",
                        -5
                )
            }
            PosStatus.connectFail -> {
                return resultContent(
                        false,
                        "pos机连接失败",
                        -7,
                        orderNumber,
                        false,
                        "pos机连接失败",
                        -7
                )
            }
            PosStatus.fail -> {
                return resultContent(
                        false,
                        "pos机连接失败",
                        -7,
                        orderNumber,
                        false,
                        "pos机连接失败",
                        -7
                )
            }
            else -> {
                return resultContent(
                        false,
                        "取消",
                        -4,
                        orderNumber,
                        false,
                        "取消",
                        -4
                )
            }
        }
    }
    //获取订单失败返回结果
    fun getOrderFailResult(order:CardPayOrder): String{
        return resultContent(
                false,
                "无法获取订单信息",
                -8,
                order.orderNumber,
                false,
                "无法获取订单信息",
                -8
        )
    }

    //参数有误
    fun getParameterErrorResult(): String{
        return resultContent(
                false,
                "参数错误，无法获取订单",
                -8,
                "",
                false,
                "参数错误，无法获取订单",
                -8
        )
    }
    //参数有误
    fun getPosPayFailResult(): String{
        return resultContent(
                false,
                "pos机连接失败",
                -7,
                "",
                false,
                "pos机连接失败",
                -7
        )
    }

    //获取订单状态错误返回结果
    fun getErrorResult(order:CardPayOrder): String{
        return resultContent(
                false,
                "订单状态不正确:${order.status}",
                -3,
                order.orderNumber,
                false,
                "订单状态不正确:${order.status}",
                -3
        )
    }


    fun resultContent(isSucc: Boolean, errorMessage: String, errorCode: Int, orderNumber: String, paySucc: Boolean, payErrorMessage: String, payErrorCode: Int):String {
        return jsObject {
            "is_succ" /= isSucc
            "error_message" /= errorMessage
            "error_code" /= errorCode
            "orderNumber" /= orderNumber
            "pay_succ" /= paySucc
            "pay_error_message" /= payErrorMessage
            "pay_error_code" /= payErrorCode
        }.toString()
    }
}