package routes.pos

import com.mjitech.api.OrderAPI
import config.AppConfig
import io.ktor.application.call
import io.ktor.request.receiveOrNull
import io.ktor.response.respond
import io.ktor.routing.Route
import io.ktor.routing.post
import io.ktor.routing.route
import mu.KotlinLogging
import org.koin.ktor.ext.inject
import routes.pos.PosResult.getErrorResult
import routes.pos.PosResult.getOrderFailResult
import routes.pos.PosResult.getParameterErrorResult
import routes.pos.PosResult.getPosPayFailResult
import routes.pos.PosResult.getResultByCode

private val logger = KotlinLogging.logger { }

fun Route.posRoutes(appconfig: AppConfig) {
    val orderAPI: OrderAPI by inject()

    route("/maxbox_pc/local_api/pay_with_pos.action") {
        post {4
            val text:String?= call.receiveOrNull()
            if (text.isNullOrBlank()) {
                call.respond(getParameterErrorResult())
            } else {
                val response = orderAPI.getOrderForCardPay(text)
                response.onSuccess { (isSuccess, res) ->
                    val order = res.order
                    val orderNumber = order.orderNumber
                    if (isSuccess) {
                        if (order.status != 1 && order.status != 66) {
                            logger.warn { "订单状态不正确" }
                            call.respond(getErrorResult(order))
                        } else {
                            val cardPayBusinessType = appconfig.cardPayBusinessType
                            val payResponse = orderAPI.connectPos(cardPayBusinessType, order, res.printContent)
                            payResponse.onSuccess { payRes ->
                                call.respond(getResultByCode(payRes, orderAPI, orderNumber))
                            }
                        }
                    } else {
                        call.respond(getOrderFailResult(order))
                    }
                }
                response.onFailure {exception ->
                    logger.error("pos pay:${text} has error", exception)
                    call.respond(getPosPayFailResult())
                }

            }
        }
    }
}



