package routes.test

import com.fasterxml.jackson.databind.JsonNode
import com.mjitech.api.model.SKU
import com.mjitech.iot.DeviceStatus
import com.mjitech.api.model.Order
import com.mjitech.api.model.OrderBatch
import com.mjitech.service.state.StateManager
import com.mjitech.service.state.entity.Event
import com.mjitech.third.scancode.ScanCodeChannel
import io.ktor.application.call
import io.ktor.http.HttpStatusCode
import io.ktor.http.content.resolveResource
import io.ktor.request.receive
import io.ktor.response.respond
import io.ktor.routing.Route
import io.ktor.routing.get
import io.ktor.routing.post
import io.ktor.routing.route
import mu.KLogger
import mu.KotlinLogging
import org.koin.ktor.ext.inject

fun Route.testRoutes() {
    val stateManager: StateManager by inject()
    val scanCodeChannel by inject<ScanCodeChannel>()
    val logger: KLogger = KotlinLogging.logger { }

    val goods1 = SKU(
            listOf(),
            "",
            113567,
            13,
            1,
            0,
            0,
            6,
            false,
            65,
            230,
            "65",
            "怡宝饮用纯净水",
            0,
            1,
            0,
            71,
            2,
            2,
            971,
            1
    )
    val goods2 = SKU(
            listOf(),
            "",
            113566,
            13,
            1,
            0,
            0,
            6,
            false,
            65,
            230,
            "65",
            "怡宝饮用纯净水",
            0,
            1,
            0,
            71,
            0,
            2,
            972,
            1
    )

    val orderSuccess = Order(mutableListOf(OrderBatch(123, listOf(goods1,goods1),1,1,1,1,1,1,1,1)),"4103",1
//            "SO0007",
//            4103,
//            listOf(goods1,goods1)
    )

//    val orderFail = Order(
//            "SO0007",
//            4103,
//            listOf(goods1,goods2)
//    )



    route("/debugIndex") {
        get {
            call.respond(call.resolveResource("test/debug.html")!!)
        }
    }
    route("/jquery") {
        get {
            call.respond(call.resolveResource("test/jquery-3.5.1.min.js")!!)
        }
    }

    route("/testwrong") {
        post {
            val body = call.receive<JsonNode>()
            val type = body["type"].asText().toInt()
            val takingNumber =  body["takingNumber"].asText()
            logger.info { "test debugShow type:$type---takingNumber:$takingNumber" }
            when(type){
                //恢复
                DebugType.free ->{
                    stateManager.handleEvent(Event.TestSetState(DeviceStatus.Free,orderSuccess))
                }
                //机器错误
                DebugType.machineError ->{
                    stateManager.handleEvent(Event.TestSetState(DeviceStatus.DownMachineLost,orderSuccess))
                }
                //正常出货
                DebugType.outGoodSuccess ->{
                    stateManager.handleEvent(Event.TestSetState(DeviceStatus.TakingGood,orderSuccess))
                }
                //有缺失货物
                DebugType.outGoodFail ->{
                    stateManager.handleEvent(Event.TestSetState(DeviceStatus.TakeGoodError,orderSuccess))
                }
                //开门
                DebugType.openGoodDoor ->{
                    stateManager.handleEvent(Event.TestSetState(DeviceStatus.OpenGoodDoor,orderSuccess))
                }
                //取货码取货
                DebugType.takingNumber ->{
                    scanCodeChannel.send(takingNumber)
                }
                //系统升级
                DebugType.updateMachine ->{
                    stateManager.handleEvent(Event.TestSetState(DeviceStatus.UpdateInReapire,orderSuccess))
                }
                else ->{

                }

            }
            call.respond(HttpStatusCode.OK)
        }
    }
}
