package com.mjitech.iot

import com.fasterxml.jackson.databind.ObjectMapper
import com.mjitech.service.mqtt.command.DataBody
import com.mjitech.service.state.entity.Event
import com.timzaak.jackson.jsObject
import com.timzaak.ucloud.mqtt.UMqttClient
import com.timzaak.ucloud.mqtt.entity.ResponseCommand
import com.typesafe.config.Config
import helper.mqttRule
import kotlinx.coroutines.channels.consumeEach
import kotlinx.coroutines.launch
import org.junit.Rule
import org.junit.Test
import org.koin.test.KoinTest
import org.koin.test.inject
import java.text.SimpleDateFormat
import java.util.*


class IotClientTest : KoinTest {

    @get:Rule
    val rule = mqttRule()
    val uMqttClient: UMqttClient by inject()
    val serverClient: IotClient by inject()
    val objectMapper: ObjectMapper by inject()
    val config: Config by inject()


    @Test
    fun getDeviceData() {
        val deviceId = config.getString("api.mqtt.deviceSN")
        val result = serverClient.getDeviceData(deviceId)
        if (result == null) {
            println("send message error")
        } else {
            val msg = result.deviceInfoSet.password
            println("send message result: $msg")
        }
    }

    @Test
    fun getDeviceInfo() {
        val deviceId = config.getString("api.mqtt.deviceSN")
        val result = serverClient.getDeviceInfo(deviceId)
        if (result == null) {
            println("send message error")
        } else {
            val msg = result.property.get("images")
            println("send message result: $msg")
        }
    }

    @Test
    fun NewSkuPass() {
        val deviceId = config.getString("api.mqtt.deviceSN")
        val result = serverClient.NewSkuPass(deviceId,1,60,117000,30,3)
        if (result == null) {
            println("send message error")
        } else {
            println("send message result: $result")
        }
    }

    @Test
    fun openDoorTest() {
        val deviceId = config.getString("api.mqtt.deviceSN")
        uMqttClient.mqttMessageChannel.launch {
            uMqttClient.mqttMessageChannel.channel.openSubscription().consumeEach { (topic, message) ->
                println("receive topic: $topic")
                println("receive message: $message")
                val json = objectMapper.readTree(message.payload)
                val msg = objectMapper.convertValue(json, DataBody::class.java)
                val input = objectMapper.convertValue(msg.Input, Event.OpenDoorInRepair::class.java)
                println("receive msg: $msg")
                println("receive Input: $input")
                val output = jsObject {
                    "isOpen" /= 1
                }.asTree()
                val responseCommand = ResponseCommand(msg.RequestID, msg.Identifier, output as com.fasterxml.jackson.databind.node.ObjectNode, 0)
                uMqttClient.respCommand(responseCommand)
                // todo: response
            }
        }
        Thread.sleep(5 * 1000)
        serverClient.openDoor(deviceId, DoorId.ADoor,45)
//        serverClient.setState(deviceId, DeviceStatus.InRepair)
        Thread.sleep(5 * 1000)
    }
}
